"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.eject = eject;
exports.getErrors = getErrors;
exports.isValid = isValid;
exports.ValidationProxy = ValidationProxy;

var _pureValidations = require("@totalsoft/pure-validations");

var isValidPropSymbol = Symbol("isValid");
var errorsPropSymbol = Symbol("errors");
var errorSeparatorSymbol = Symbol("errorSeparator");
var targetSymbol = Symbol("target");
var defaultErrorSeparator = ", ";
var handler = {
  get: function get(target, name) {
    if (name in target) {
      return target[name];
    }

    switch (name) {
      case targetSymbol:
        {
          return target;
        }

      case isValidPropSymbol:
        {
          var valid = (0, _pureValidations.isValid)(target);
          target[isValidPropSymbol] = valid; // cache value

          return valid;
        }

      case errorsPropSymbol:
        {
          var errors = (0, _pureValidations.getErrors)(target).join(target[errorSeparatorSymbol] || defaultErrorSeparator);
          target[errorsPropSymbol] = errors; // cache value

          return errors;
        }

      default:
        {
          var proxy = ValidationProxy((0, _pureValidations.getInner)([name], target));
          target[name] = proxy; // cache value

          return proxy;
        }
    }
  }
};
var successProxy = initializeSuccessProxy();

function initializeSuccessProxy() {
  var proxy = new Proxy(_pureValidations.Success, handler);
  proxy[isValidPropSymbol];
  proxy[errorsPropSymbol];
  proxy[targetSymbol];
  return proxy;
}

function eject(proxy) {
  return proxy[targetSymbol];
}

function getErrors(proxy, separator) {
  proxy[errorSeparatorSymbol] = separator;
  return proxy[errorsPropSymbol];
}

function isValid(proxy) {
  return proxy[isValidPropSymbol];
}

function ValidationProxy(validation) {
  return isValid(validation) ? successProxy : new Proxy(validation, handler);
}